@@ -1041,12 +1041,12 @@ void Bind2Backend::queueReloadAndStore(unsigned int id)
   try {
     if (!safeGetBBDomainInfo(id, &bbold))
       return;
+    bbold.d_checknow = false;
     BB2DomainInfo bbnew(bbold);
     /* make sure that nothing will be able to alter the existing records,
        we will load them from the zone file instead */
     bbnew.d_records = LookButDontTouch<recordstorage_t>();
     parseZoneFile(&bbnew);
-    bbnew.d_checknow = false;
     bbnew.d_wasRejectedLastReload = false;
     safePutBBDomainInfo(bbnew);
     g_log << Logger::Warning << "Zone '" << bbnew.d_name << "' (" << bbnew.d_filename << ") reloaded" << endl;
@@ -1056,6 +1056,7 @@ void Bind2Backend::queueReloadAndStore(unsigned int id)
     msg << " error at " + nowTime() + " parsing '" << bbold.d_name << "' from file '" << bbold.d_filename << "': " << ae.reason;
     g_log << Logger::Warning << " error parsing '" << bbold.d_name << "' from file '" << bbold.d_filename << "': " << ae.reason << endl;
     bbold.d_status = msg.str();
+    bbold.d_lastcheck = time(nullptr);
     bbold.d_wasRejectedLastReload = true;
     safePutBBDomainInfo(bbold);
   }
@@ -1064,6 +1065,7 @@ void Bind2Backend::queueReloadAndStore(unsigned int id)
     msg << " error at " + nowTime() + " parsing '" << bbold.d_name << "' from file '" << bbold.d_filename << "': " << ae.what();
     g_log << Logger::Warning << " error parsing '" << bbold.d_name << "' from file '" << bbold.d_filename << "': " << ae.what() << endl;
     bbold.d_status = msg.str();
+    bbold.d_lastcheck = time(nullptr);
     bbold.d_wasRejectedLastReload = true;
     safePutBBDomainInfo(bbold);
   }
@@ -1179,11 +1181,6 @@ void Bind2Backend::lookup(const QType& qtype, const DNSName& qname, int zoneId,
   d_handle.qtype = qtype;
   d_handle.domain = std::move(domain);
 
-  if (!bbd.d_loaded) {
-    d_handle.reset();
-    throw DBException("Zone for '" + d_handle.domain.toLogString() + "' in '" + bbd.d_filename + "' temporarily not available (file missing, or master dead)"); // fsck
-  }
-
   if (!bbd.current()) {
     g_log << Logger::Warning << "Zone '" << d_handle.domain << "' (" << bbd.d_filename << ") needs reloading" << endl;
     queueReloadAndStore(bbd.d_id);
@@ -1191,6 +1188,11 @@ void Bind2Backend::lookup(const QType& qtype, const DNSName& qname, int zoneId,
       throw DBException("Zone '" + bbd.d_name.toLogString() + "' (" + bbd.d_filename + ") gone after reload"); // if we don't throw here, we crash for some reason
   }
 
+  if (!bbd.d_loaded) {
+    d_handle.reset();
+    throw DBException("Zone for '" + d_handle.domain.toLogString() + "' in '" + bbd.d_filename + "' not loaded (file missing, corrupt or master dead)"); // fsck
+  }
+
   d_handle.d_records = bbd.d_records.get();
 
   if (d_handle.d_records->empty())
@@ -1299,6 +1301,10 @@ bool Bind2Backend::list(const DNSName& target, int id, bool include_disabled)
   d_handle.reset();
   DLOG(g_log << "Bind2Backend constructing handle for list of " << id << endl);
 
+  if (!bbd.d_loaded) {
+    throw PDNSException("zone was not loaded, perhaps because of: " + bbd.d_status);
+  }
+
   d_handle.d_records = bbd.d_records.get(); // give it a copy, which will stay around
   d_handle.d_qname_iter = d_handle.d_records->begin();
   d_handle.d_qname_end = d_handle.d_records->end(); // iter now points to a vector of pointers to vector<BBResourceRecords>
@@ -1435,6 +1441,10 @@ bool Bind2Backend::searchRecords(const string& pattern, int maxResults, vector<D
         continue;
       }
 
+      if (!h.d_loaded) {
+        continue;
+      }
+
       shared_ptr<const recordstorage_t> rhandle = h.d_records.get();
 
       for (recordstorage_t::const_iterator ri = rhandle->begin(); result.size() < static_cast<vector<DNSResourceRecord>::size_type>(maxResults) && ri != rhandle->end(); ri++) {
