@@ -31,82 +31,128 @@
 
 void Bind2Backend::setupDNSSEC()
 {
-  if(!getArg("dnssec-db").empty())
+  if (!getArg("dnssec-db").empty())
     throw runtime_error("bind-dnssec-db requires building PowerDNS with SQLite3");
 }
 
 bool Bind2Backend::doesDNSSEC()
-{ return d_hybrid; }
+{
+  return d_hybrid;
+}
 
 bool Bind2Backend::getNSEC3PARAM(const DNSName& name, NSEC3PARAMRecordContent* ns3p)
-{ return false; }
+{
+  return false;
+}
 
-bool Bind2Backend::getAllDomainMetadata(const DNSName& name, std::map<std::string, std::vector<std::string> >& meta)
-{ return false; }
+bool Bind2Backend::getNSEC3PARAMuncached(const DNSName& name, NSEC3PARAMRecordContent* ns3p)
+{
+  return false;
+}
+
+bool Bind2Backend::getAllDomainMetadata(const DNSName& name, std::map<std::string, std::vector<std::string>>& meta)
+{
+  return false;
+}
 
 bool Bind2Backend::getDomainMetadata(const DNSName& name, const std::string& kind, std::vector<std::string>& meta)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::setDomainMetadata(const DNSName& name, const std::string& kind, const std::vector<std::string>& meta)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::getDomainKeys(const DNSName& name, std::vector<KeyData>& keys)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::removeDomainKey(const DNSName& name, unsigned int id)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::addDomainKey(const DNSName& name, const KeyData& key, int64_t& id)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::activateDomainKey(const DNSName& name, unsigned int id)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::deactivateDomainKey(const DNSName& name, unsigned int id)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::publishDomainKey(const DNSName& name, unsigned int id)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::unpublishDomainKey(const DNSName& name, unsigned int id)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::getTSIGKey(const DNSName& name, DNSName* algorithm, string* content)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::setTSIGKey(const DNSName& name, const DNSName& algorithm, const string& content)
-{ return false; }
+{
+  return false;
+}
 
 bool Bind2Backend::deleteTSIGKey(const DNSName& name)
-{ return false; }
+{
+  return false;
+}
 
-bool Bind2Backend::getTSIGKeys(std::vector<struct TSIGKey> &keys)
-{ return false; }
+bool Bind2Backend::getTSIGKeys(std::vector<struct TSIGKey>& keys)
+{
+  return false;
+}
 
 void Bind2Backend::setupStatements()
-{ return; }
+{
+  return;
+}
 
 void Bind2Backend::freeStatements()
-{ return; }
+{
+  return;
+}
 
 #else
 
 #include "pdns/logger.hh"
 #include "pdns/ssqlite3.hh"
 
-#define ASSERT_ROW_COLUMNS(query, row, num) { if (row.size() != num) { throw PDNSException(std::string(query) + " returned wrong number of columns, expected "  #num  ", got " + std::to_string(row.size())); } }
+#define ASSERT_ROW_COLUMNS(query, row, num)                                                                                                \
+  {                                                                                                                                        \
+    if (row.size() != num) {                                                                                                               \
+      throw PDNSException(std::string(query) + " returned wrong number of columns, expected " #num ", got " + std::to_string(row.size())); \
+    }                                                                                                                                      \
+  }
 
 void Bind2Backend::setupDNSSEC()
 {
-  if(getArg("dnssec-db").empty() || d_hybrid)
+  if (getArg("dnssec-db").empty() || d_hybrid)
     return;
   try {
     d_dnssecdb = shared_ptr<SSQLite3>(new SSQLite3(getArg("dnssec-db"), getArg("dnssec-db-journal-mode")));
     setupStatements();
   }
-  catch(SSqlException& se) {
+  catch (SSqlException& se) {
     // this error is meant to kill the server dead - it makes no sense to continue..
-    throw runtime_error("Error opening DNSSEC database in BIND backend: "+se.txtReason());
+    throw runtime_error("Error opening DNSSEC database in BIND backend: " + se.txtReason());
   }
 
   d_dnssecdb->setLog(::arg().mustDo("query-logging"));
@@ -114,12 +160,12 @@ void Bind2Backend::setupDNSSEC()
 
 void Bind2Backend::setupStatements()
 {
-  d_getAllDomainMetadataQuery_stmt = d_dnssecdb->prepare("select kind, content from domainmetadata where domain=:domain",1);
-  d_getDomainMetadataQuery_stmt = d_dnssecdb->prepare("select content from domainmetadata where domain=:domain and kind=:kind",2);
-  d_deleteDomainMetadataQuery_stmt = d_dnssecdb->prepare("delete from domainmetadata where domain=:domain and kind=:kind",2);
-  d_insertDomainMetadataQuery_stmt = d_dnssecdb->prepare("insert into domainmetadata (domain, kind, content) values (:domain,:kind,:content)",3);
-  d_getDomainKeysQuery_stmt = d_dnssecdb->prepare("select id,flags, active, published, content from cryptokeys where domain=:domain",1);
-  d_deleteDomainKeyQuery_stmt = d_dnssecdb->prepare("delete from cryptokeys where domain=:domain and id=:key_id",2);
+  d_getAllDomainMetadataQuery_stmt = d_dnssecdb->prepare("select kind, content from domainmetadata where domain=:domain", 1);
+  d_getDomainMetadataQuery_stmt = d_dnssecdb->prepare("select content from domainmetadata where domain=:domain and kind=:kind", 2);
+  d_deleteDomainMetadataQuery_stmt = d_dnssecdb->prepare("delete from domainmetadata where domain=:domain and kind=:kind", 2);
+  d_insertDomainMetadataQuery_stmt = d_dnssecdb->prepare("insert into domainmetadata (domain, kind, content) values (:domain,:kind,:content)", 3);
+  d_getDomainKeysQuery_stmt = d_dnssecdb->prepare("select id,flags, active, published, content from cryptokeys where domain=:domain", 1);
+  d_deleteDomainKeyQuery_stmt = d_dnssecdb->prepare("delete from cryptokeys where domain=:domain and id=:key_id", 2);
   d_insertDomainKeyQuery_stmt = d_dnssecdb->prepare("insert into cryptokeys (domain, flags, active, published, content) values (:domain, :flags, :active, :published, :content)", 5);
   d_GetLastInsertedKeyIdQuery_stmt = d_dnssecdb->prepare("select last_insert_rowid()", 0);
   d_activateDomainKeyQuery_stmt = d_dnssecdb->prepare("update cryptokeys set active=1 where domain=:domain and id=:key_id", 2);
@@ -159,29 +205,42 @@ bool Bind2Backend::doesDNSSEC()
 
 bool Bind2Backend::getNSEC3PARAM(const DNSName& name, NSEC3PARAMRecordContent* ns3p)
 {
-  if(!d_dnssecdb || d_hybrid)
+  BB2DomainInfo bbd;
+  if (!safeGetBBDomainInfo(name, &bbd))
+    return false;
+
+  if (ns3p) {
+    *ns3p = bbd.d_nsec3param;
+  }
+
+  return bbd.d_nsec3zone;
+}
+
+bool Bind2Backend::getNSEC3PARAMuncached(const DNSName& name, NSEC3PARAMRecordContent* ns3p)
+{
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   string value;
   vector<string> meta;
   getDomainMetadata(name, "NSEC3PARAM", meta);
-  if(!meta.empty())
-    value=*meta.begin();
+  if (!meta.empty())
+    value = *meta.begin();
   else
     return false; // No NSEC3 zone
 
-  static int maxNSEC3Iterations=::arg().asNum("max-nsec3-iterations");
-  if(ns3p) {
-    auto tmp=std::dynamic_pointer_cast<NSEC3PARAMRecordContent>(DNSRecordContent::mastermake(QType::NSEC3PARAM, 1, value));
+  static int maxNSEC3Iterations = ::arg().asNum("max-nsec3-iterations");
+  if (ns3p) {
+    auto tmp = std::dynamic_pointer_cast<NSEC3PARAMRecordContent>(DNSRecordContent::mastermake(QType::NSEC3PARAM, 1, value));
     *ns3p = *tmp;
 
     if (ns3p->d_iterations > maxNSEC3Iterations) {
       ns3p->d_iterations = maxNSEC3Iterations;
-      g_log<<Logger::Error<<"Number of NSEC3 iterations for zone '"<<name<<"' is above 'max-nsec3-iterations'. Value adjusted to: "<<maxNSEC3Iterations<<endl;
+      g_log << Logger::Error << "Number of NSEC3 iterations for zone '" << name << "' is above 'max-nsec3-iterations'. Value adjusted to: " << maxNSEC3Iterations << endl;
     }
 
     if (ns3p->d_algorithm != 1) {
-      g_log<<Logger::Error<<"Invalid hash algorithm for NSEC3: '"<<std::to_string(ns3p->d_algorithm)<<"', setting to 1 for zone '"<<name<<"'."<<endl;
+      g_log << Logger::Error << "Invalid hash algorithm for NSEC3: '" << std::to_string(ns3p->d_algorithm) << "', setting to 1 for zone '" << name << "'." << endl;
       ns3p->d_algorithm = 1;
     }
   }
@@ -189,96 +248,80 @@ bool Bind2Backend::getNSEC3PARAM(const DNSName& name, NSEC3PARAMRecordContent* n
   return true;
 }
 
-bool Bind2Backend::getAllDomainMetadata(const DNSName& name, std::map<std::string, std::vector<std::string> >& meta)
+bool Bind2Backend::getAllDomainMetadata(const DNSName& name, std::map<std::string, std::vector<std::string>>& meta)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_getAllDomainMetadataQuery_stmt->
-      bind("domain", name)->
-      execute();
+    d_getAllDomainMetadataQuery_stmt->bind("domain", name)->execute();
 
     SSqlStatement::row_t row;
-    while(d_getAllDomainMetadataQuery_stmt->hasNextRow()) {
+    while (d_getAllDomainMetadataQuery_stmt->hasNextRow()) {
       d_getAllDomainMetadataQuery_stmt->nextRow(row);
       meta[row[0]].push_back(row[1]);
     }
 
     d_getAllDomainMetadataQuery_stmt->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, getAllDomainMetadata(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, getAllDomainMetadata(): " + se.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::getDomainMetadata(const DNSName& name, const std::string& kind, std::vector<std::string>& meta)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_getDomainMetadataQuery_stmt->
-      bind("domain", name)->
-      bind("kind", kind)->
-      execute();
+    d_getDomainMetadataQuery_stmt->bind("domain", name)->bind("kind", kind)->execute();
 
     SSqlStatement::row_t row;
-    while(d_getDomainMetadataQuery_stmt->hasNextRow()) {
+    while (d_getDomainMetadataQuery_stmt->hasNextRow()) {
       d_getDomainMetadataQuery_stmt->nextRow(row);
       meta.push_back(row[0]);
     }
 
     d_getDomainMetadataQuery_stmt->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, getDomainMetadata(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, getDomainMetadata(): " + se.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::setDomainMetadata(const DNSName& name, const std::string& kind, const std::vector<std::string>& meta)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_deleteDomainMetadataQuery_stmt->
-      bind("domain", name)->
-      bind("kind", kind)->
-      execute()->
-      reset();
-    if(!meta.empty()) {
-      for(const auto& value: meta) {
-        d_insertDomainMetadataQuery_stmt->
-          bind("domain", name)->
-          bind("kind", kind)->
-          bind("content", value)->
-          execute()->
-          reset();
+    d_deleteDomainMetadataQuery_stmt->bind("domain", name)->bind("kind", kind)->execute()->reset();
+    if (!meta.empty()) {
+      for (const auto& value : meta) {
+        d_insertDomainMetadataQuery_stmt->bind("domain", name)->bind("kind", kind)->bind("content", value)->execute()->reset();
       }
     }
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, setDomainMetadata(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, setDomainMetadata(): " + se.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::getDomainKeys(const DNSName& name, std::vector<KeyData>& keys)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_getDomainKeysQuery_stmt->
-      bind("domain", name)->
-      execute();
+    d_getDomainKeysQuery_stmt->bind("domain", name)->execute();
 
     KeyData kd;
     SSqlStatement::row_t row;
-    while(d_getDomainKeysQuery_stmt->hasNextRow()) {
+    while (d_getDomainKeysQuery_stmt->hasNextRow()) {
       d_getDomainKeysQuery_stmt->nextRow(row);
       kd.id = pdns_stou(row[0]);
       kd.flags = pdns_stou(row[1]);
@@ -290,47 +333,36 @@ bool Bind2Backend::getDomainKeys(const DNSName& name, std::vector<KeyData>& keys
 
     d_getDomainKeysQuery_stmt->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, getDomainKeys(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, getDomainKeys(): " + se.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::removeDomainKey(const DNSName& name, unsigned int id)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_deleteDomainKeyQuery_stmt->
-      bind("domain", name)->
-      bind("key_id", id)->
-      execute()->
-      reset();
+    d_deleteDomainKeyQuery_stmt->bind("domain", name)->bind("key_id", id)->execute()->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, removeDomainKeys(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, removeDomainKeys(): " + se.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::addDomainKey(const DNSName& name, const KeyData& key, int64_t& id)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_insertDomainKeyQuery_stmt->
-      bind("domain", name)->
-      bind("flags", key.flags)->
-      bind("active", key.active)->
-      bind("published", key.published)->
-      bind("content", key.content)->
-      execute()->
-      reset();
+    d_insertDomainKeyQuery_stmt->bind("domain", name)->bind("flags", key.flags)->bind("active", key.active)->bind("published", key.published)->bind("content", key.content)->execute()->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, addDomainKey(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, addDomainKey(): " + se.txtReason());
   }
 
   try {
@@ -346,7 +378,7 @@ bool Bind2Backend::addDomainKey(const DNSName& name, const KeyData& key, int64_t
     d_GetLastInsertedKeyIdQuery_stmt->reset();
     return true;
   }
-  catch (SSqlException &e) {
+  catch (SSqlException& e) {
     id = -2;
     return true;
   }
@@ -356,92 +388,73 @@ bool Bind2Backend::addDomainKey(const DNSName& name, const KeyData& key, int64_t
 
 bool Bind2Backend::activateDomainKey(const DNSName& name, unsigned int id)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_activateDomainKeyQuery_stmt->
-      bind("domain", name)->
-      bind("key_id", id)->
-      execute()->
-      reset();
+    d_activateDomainKeyQuery_stmt->bind("domain", name)->bind("key_id", id)->execute()->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, activateDomainKey(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, activateDomainKey(): " + se.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::deactivateDomainKey(const DNSName& name, unsigned int id)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_deactivateDomainKeyQuery_stmt->
-      bind("domain", name)->
-      bind("key_id", id)->
-      execute()->
-      reset();
+    d_deactivateDomainKeyQuery_stmt->bind("domain", name)->bind("key_id", id)->execute()->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, deactivateDomainKey(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, deactivateDomainKey(): " + se.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::publishDomainKey(const DNSName& name, unsigned int id)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_publishDomainKeyQuery_stmt->
-      bind("domain", name)->
-      bind("key_id", id)->
-      execute()->
-      reset();
+    d_publishDomainKeyQuery_stmt->bind("domain", name)->bind("key_id", id)->execute()->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, publishDomainKey(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, publishDomainKey(): " + se.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::unpublishDomainKey(const DNSName& name, unsigned int id)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_unpublishDomainKeyQuery_stmt->
-      bind("domain", name)->
-      bind("key_id", id)->
-      execute()->
-      reset();
+    d_unpublishDomainKeyQuery_stmt->bind("domain", name)->bind("key_id", id)->execute()->reset();
   }
-  catch(SSqlException& se) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, unpublishDomainKey(): "+se.txtReason());
+  catch (SSqlException& se) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, unpublishDomainKey(): " + se.txtReason());
   }
   return true;
 }
 
-
 bool Bind2Backend::getTSIGKey(const DNSName& name, DNSName* algorithm, string* content)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_getTSIGKeyQuery_stmt->
-      bind("key_name", name)->
-      execute();
+    d_getTSIGKeyQuery_stmt->bind("key_name", name)->execute();
 
     SSqlStatement::row_t row;
     content->clear();
-    while(d_getTSIGKeyQuery_stmt->hasNextRow()) {
+    while (d_getTSIGKeyQuery_stmt->hasNextRow()) {
       d_getTSIGKeyQuery_stmt->nextRow(row);
-      if(row.size() >= 2 && (algorithm->empty() || *algorithm == DNSName(row[0]))) {
+      if (row.size() >= 2 && (algorithm->empty() || *algorithm == DNSName(row[0]))) {
         *algorithm = DNSName(row[0]);
         *content = row[1];
       }
@@ -449,59 +462,50 @@ bool Bind2Backend::getTSIGKey(const DNSName& name, DNSName* algorithm, string* c
 
     d_getTSIGKeyQuery_stmt->reset();
   }
-  catch (SSqlException &e) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, getTSIGKey(): "+e.txtReason());
+  catch (SSqlException& e) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, getTSIGKey(): " + e.txtReason());
   }
   return !content->empty();
 }
 
 bool Bind2Backend::setTSIGKey(const DNSName& name, const DNSName& algorithm, const string& content)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_setTSIGKeyQuery_stmt->
-      bind("key_name", name)->
-      bind("algorithm", algorithm)->
-      bind("content", content)->
-      execute()->
-      reset();
+    d_setTSIGKeyQuery_stmt->bind("key_name", name)->bind("algorithm", algorithm)->bind("content", content)->execute()->reset();
   }
-  catch (SSqlException &e) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, setTSIGKey(): "+e.txtReason());
+  catch (SSqlException& e) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, setTSIGKey(): " + e.txtReason());
   }
   return true;
 }
 
 bool Bind2Backend::deleteTSIGKey(const DNSName& name)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_deleteTSIGKeyQuery_stmt->
-      bind("key_name", name)->
-      execute()->
-      reset();
+    d_deleteTSIGKeyQuery_stmt->bind("key_name", name)->execute()->reset();
   }
-  catch (SSqlException &e) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, deleteTSIGKey(): "+e.txtReason());
+  catch (SSqlException& e) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, deleteTSIGKey(): " + e.txtReason());
   }
   return true;
 }
 
-bool Bind2Backend::getTSIGKeys(std::vector< struct TSIGKey > &keys)
+bool Bind2Backend::getTSIGKeys(std::vector<struct TSIGKey>& keys)
 {
-  if(!d_dnssecdb || d_hybrid)
+  if (!d_dnssecdb || d_hybrid)
     return false;
 
   try {
-    d_getTSIGKeysQuery_stmt->
-      execute();
+    d_getTSIGKeysQuery_stmt->execute();
 
     SSqlStatement::row_t row;
-    while(d_getTSIGKeysQuery_stmt->hasNextRow()) {
+    while (d_getTSIGKeysQuery_stmt->hasNextRow()) {
       d_getTSIGKeysQuery_stmt->nextRow(row);
       struct TSIGKey key;
       key.name = DNSName(row[0]);
@@ -512,8 +516,8 @@ bool Bind2Backend::getTSIGKeys(std::vector< struct TSIGKey > &keys)
 
     d_getTSIGKeysQuery_stmt->reset();
   }
-  catch (SSqlException &e) {
-    throw PDNSException("Error accessing DNSSEC database in BIND backend, getTSIGKeys(): "+e.txtReason());
+  catch (SSqlException& e) {
+    throw PDNSException("Error accessing DNSSEC database in BIND backend, getTSIGKeys(): " + e.txtReason());
   }
   return !keys.empty();
 }
