@@ -49,13 +49,15 @@ typedef bool my_bool;
 class MySQLThreadCloser
 {
 public:
-  ~MySQLThreadCloser() {
-    if(d_enabled) {
+  ~MySQLThreadCloser()
+  {
+    if (d_enabled) {
       mysql_thread_end();
     }
   }
-  void enable() {
-   d_enabled = true;
+  void enable()
+  {
+    d_enabled = true;
   }
 
 private:
@@ -67,25 +69,27 @@ static thread_local MySQLThreadCloser threadcloser;
 bool SMySQL::s_dolog;
 std::mutex SMySQL::s_myinitlock;
 
-class SMySQLStatement: public SSqlStatement
+class SMySQLStatement : public SSqlStatement
 {
 public:
-  SMySQLStatement(const string& query, bool dolog, int nparams, MYSQL* db) : d_prepared(false)
+  SMySQLStatement(const string& query, bool dolog, int nparams, MYSQL* db) :
+    d_prepared(false)
   {
     d_db = db;
     d_dolog = dolog;
     d_query = query;
     d_paridx = d_fnum = d_resnum = d_residx = 0;
     d_parnum = nparams;
-    d_req_bind = d_res_bind = NULL;
-    d_stmt = NULL;
+    d_req_bind = d_res_bind = nullptr;
+    d_stmt = nullptr;
 
     if (query.empty()) {
       return;
     }
   }
 
-  SSqlStatement* bind(const string& name, bool value) {
+  SSqlStatement* bind(const string& name, bool value)
+  {
     prepareStatement();
     if (d_paridx >= d_parnum) {
       releaseStatement();
@@ -93,17 +97,20 @@ public:
     }
     d_req_bind[d_paridx].buffer_type = MYSQL_TYPE_TINY;
     d_req_bind[d_paridx].buffer = new char[1];
-    *((char*)d_req_bind[d_paridx].buffer) = (value?1:0);
+    *((char*)d_req_bind[d_paridx].buffer) = (value ? 1 : 0);
     d_paridx++;
     return this;
   }
-  SSqlStatement* bind(const string& name, int value) {
+  SSqlStatement* bind(const string& name, int value)
+  {
     return bind(name, (long)value);
   }
-  SSqlStatement* bind(const string& name, uint32_t value) {
+  SSqlStatement* bind(const string& name, uint32_t value)
+  {
     return bind(name, (unsigned long)value);
   }
-  SSqlStatement* bind(const string& name, long value) {
+  SSqlStatement* bind(const string& name, long value)
+  {
     prepareStatement();
     if (d_paridx >= d_parnum) {
       releaseStatement();
@@ -115,7 +122,8 @@ public:
     d_paridx++;
     return this;
   }
-  SSqlStatement* bind(const string& name, unsigned long value) {
+  SSqlStatement* bind(const string& name, unsigned long value)
+  {
     prepareStatement();
     if (d_paridx >= d_parnum) {
       releaseStatement();
@@ -128,7 +136,8 @@ public:
     d_paridx++;
     return this;
   }
-  SSqlStatement* bind(const string& name, long long value) {
+  SSqlStatement* bind(const string& name, long long value)
+  {
     prepareStatement();
     if (d_paridx >= d_parnum) {
       releaseStatement();
@@ -140,7 +149,8 @@ public:
     d_paridx++;
     return this;
   }
-  SSqlStatement* bind(const string& name, unsigned long long value) {
+  SSqlStatement* bind(const string& name, unsigned long long value)
+  {
     prepareStatement();
     if (d_paridx >= d_parnum) {
       releaseStatement();
@@ -153,23 +163,25 @@ public:
     d_paridx++;
     return this;
   }
-  SSqlStatement* bind(const string& name, const std::string& value) {
+  SSqlStatement* bind(const string& name, const std::string& value)
+  {
     prepareStatement();
     if (d_paridx >= d_parnum) {
       releaseStatement();
       throw SSqlException("Attempt to bind more parameters than query has: " + d_query);
     }
     d_req_bind[d_paridx].buffer_type = MYSQL_TYPE_STRING;
-    d_req_bind[d_paridx].buffer = new char[value.size()+1];
+    d_req_bind[d_paridx].buffer = new char[value.size() + 1];
     d_req_bind[d_paridx].length = new unsigned long[1];
     *d_req_bind[d_paridx].length = value.size();
-    d_req_bind[d_paridx].buffer_length = *d_req_bind[d_paridx].length+1;
-    memset(d_req_bind[d_paridx].buffer, 0, value.size()+1);
+    d_req_bind[d_paridx].buffer_length = *d_req_bind[d_paridx].length + 1;
+    memset(d_req_bind[d_paridx].buffer, 0, value.size() + 1);
     value.copy((char*)d_req_bind[d_paridx].buffer, value.size());
     d_paridx++;
     return this;
   }
-  SSqlStatement* bindNull(const string& name) { 
+  SSqlStatement* bindNull(const string& name)
+  {
     prepareStatement();
     if (d_paridx >= d_parnum) {
       releaseStatement();
@@ -180,13 +192,15 @@ public:
     return this;
   }
 
-  SSqlStatement* execute() {
+  SSqlStatement* execute()
+  {
     prepareStatement();
 
-    if (!d_stmt) return this;
+    if (!d_stmt)
+      return this;
 
     if (d_dolog) {
-      g_log<<Logger::Warning<< "Query "<<((long)(void*)this)<<": " << d_query << endl;
+      g_log << Logger::Warning << "Query " << ((long)(void*)this) << ": " << d_query << endl;
       d_dtime.set();
     }
 
@@ -209,20 +223,21 @@ public:
       throw SSqlException("Could not store mysql statement: " + d_query + string(": ") + error);
     }
 
-    if ((d_fnum = static_cast<int>(mysql_stmt_field_count(d_stmt)))>0) {
+    if ((d_fnum = static_cast<int>(mysql_stmt_field_count(d_stmt))) > 0) {
       // prepare for result
       d_resnum = mysql_stmt_num_rows(d_stmt);
-      
+
       if (d_resnum > 0 && d_res_bind == nullptr) {
         MYSQL_RES* meta = mysql_stmt_result_metadata(d_stmt);
         d_fnum = static_cast<int>(mysql_num_fields(meta)); // ensure correct number of fields
         d_res_bind = new MYSQL_BIND[d_fnum];
-        memset(d_res_bind, 0, sizeof(MYSQL_BIND)*d_fnum);
+        memset(d_res_bind, 0, sizeof(MYSQL_BIND) * d_fnum);
         MYSQL_FIELD* fields = mysql_fetch_fields(meta);
 
-        for(int i = 0; i < d_fnum; i++) {
-          unsigned long len = std::max(fields[i].max_length, fields[i].length)+1;
-          if (len > 128 * 1024) len = 128 * 1024; // LONGTEXT may tell us it needs 4GB!
+        for (int i = 0; i < d_fnum; i++) {
+          unsigned long len = std::max(fields[i].max_length, fields[i].length) + 1;
+          if (len > 128 * 1024)
+            len = 128 * 1024; // LONGTEXT may tell us it needs 4GB!
           d_res_bind[i].is_null = new my_bool[1];
           d_res_bind[i].error = new my_bool[1];
           d_res_bind[i].length = new unsigned long[1];
@@ -230,7 +245,7 @@ public:
           d_res_bind[i].buffer_length = len;
           d_res_bind[i].buffer_type = MYSQL_TYPE_STRING;
         }
-  
+
         mysql_free_result(meta);
       }
 
@@ -246,20 +261,22 @@ public:
       }
     }
 
-    if(d_dolog) 
-      g_log<<Logger::Warning<< "Query "<<((long)(void*)this)<<": "<<d_dtime.udiffNoReset()<<" usec to execute"<<endl;
+    if (d_dolog)
+      g_log << Logger::Warning << "Query " << ((long)(void*)this) << ": " << d_dtime.udiffNoReset() << " usec to execute" << endl;
 
     return this;
   }
 
-  bool hasNextRow() {
-    if(d_dolog && d_residx == d_resnum) {
-      g_log<<Logger::Warning<< "Query "<<((long)(void*)this)<<": "<<d_dtime.udiffNoReset()<<" total usec to last row"<<endl;
+  bool hasNextRow()
+  {
+    if (d_dolog && d_residx == d_resnum) {
+      g_log << Logger::Warning << "Query " << ((long)(void*)this) << ": " << d_dtime.udiffNoReset() << " total usec to last row" << endl;
     }
     return d_residx < d_resnum;
   }
 
-  SSqlStatement* nextRow(row_t& row) {
+  SSqlStatement* nextRow(row_t& row)
+  {
     int err;
     row.clear();
     if (!hasNextRow()) {
@@ -276,14 +293,15 @@ public:
 
     row.reserve(d_fnum);
 
-    for(int i=0;i<d_fnum;i++) {
+    for (int i = 0; i < d_fnum; i++) {
       if (err == MYSQL_DATA_TRUNCATED && *d_res_bind[i].error) {
-        g_log<<Logger::Warning<<"Result field at row " << d_residx << " column " << i << " has been truncated, we allocated " << d_res_bind[i].buffer_length << " bytes but at least " << *d_res_bind[i].length << " was needed" << endl;
+        g_log << Logger::Warning << "Result field at row " << d_residx << " column " << i << " has been truncated, we allocated " << d_res_bind[i].buffer_length << " bytes but at least " << *d_res_bind[i].length << " was needed" << endl;
       }
       if (*d_res_bind[i].is_null) {
         row.emplace_back("");
         continue;
-      } else {
+      }
+      else {
         row.emplace_back((char*)d_res_bind[i].buffer, std::min(d_res_bind[i].buffer_length, *d_res_bind[i].length));
       }
     }
@@ -292,7 +310,7 @@ public:
 #if MYSQL_VERSION_ID >= 50500
     if (d_residx >= d_resnum) {
       mysql_stmt_free_result(d_stmt);
-      while(!mysql_stmt_next_result(d_stmt)) {
+      while (!mysql_stmt_next_result(d_stmt)) {
         if (mysql_stmt_store_result(d_stmt) != 0) {
           string error(mysql_stmt_error(d_stmt));
           releaseStatement();
@@ -317,43 +335,48 @@ public:
       }
     }
 #endif
-    return this; 
+    return this;
   }
 
-  SSqlStatement* getResult(result_t& result) { 
+  SSqlStatement* getResult(result_t& result)
+  {
     result.clear();
     result.reserve(d_resnum);
     row_t row;
 
-    while(hasNextRow()) {
+    while (hasNextRow()) {
       nextRow(row);
       result.push_back(std::move(row));
     }
 
-    return this; 
+    return this;
   }
 
-  SSqlStatement* reset() {
-    if (!d_stmt) return this;
-    int err=0;
+  SSqlStatement* reset()
+  {
+    if (!d_stmt)
+      return this;
+    int err = 0;
     mysql_stmt_free_result(d_stmt);
 #if MYSQL_VERSION_ID >= 50500
-    while((err = mysql_stmt_next_result(d_stmt)) == 0) {
+    while ((err = mysql_stmt_next_result(d_stmt)) == 0) {
       mysql_stmt_free_result(d_stmt);
     }
 #endif
-    if (err>0) {
+    if (err > 0) {
       string error(mysql_stmt_error(d_stmt));
       releaseStatement();
       throw SSqlException("Could not get next result from mysql statement: " + d_query + string(": ") + error);
     }
     mysql_stmt_reset(d_stmt);
     if (d_req_bind) {
-      for(int i=0;i<d_parnum;i++) {
-        if (d_req_bind[i].buffer) delete [] (char*)d_req_bind[i].buffer;
-        if (d_req_bind[i].length) delete [] d_req_bind[i].length;
+      for (int i = 0; i < d_parnum; i++) {
+        if (d_req_bind[i].buffer)
+          delete[](char*) d_req_bind[i].buffer;
+        if (d_req_bind[i].length)
+          delete[] d_req_bind[i].length;
       }
-      memset(d_req_bind, 0, sizeof(MYSQL_BIND)*d_parnum);
+      memset(d_req_bind, 0, sizeof(MYSQL_BIND) * d_parnum);
     }
     d_residx = d_resnum = 0;
     d_paridx = 0;
@@ -362,19 +385,22 @@ public:
 
   const std::string& getQuery() { return d_query; }
 
-  ~SMySQLStatement() {
+  ~SMySQLStatement()
+  {
     releaseStatement();
   }
-private:
 
-  void prepareStatement() {
-    if (d_prepared) return;
+private:
+  void prepareStatement()
+  {
+    if (d_prepared)
+      return;
     if (d_query.empty()) {
       d_prepared = true;
       return;
     }
 
-    if ((d_stmt = mysql_stmt_init(d_db))==NULL)
+    if ((d_stmt = mysql_stmt_init(d_db)) == nullptr)
       throw SSqlException("Could not initialize mysql statement, out of memory: " + d_query);
 
     if (mysql_stmt_prepare(d_stmt, d_query.c_str(), d_query.size()) != 0) {
@@ -388,36 +414,43 @@ private:
       throw SSqlException("Provided parameter count does not match statement: " + d_query);
     }
 
-    if (d_parnum>0) {
+    if (d_parnum > 0) {
       d_req_bind = new MYSQL_BIND[d_parnum];
-      memset(d_req_bind, 0, sizeof(MYSQL_BIND)*d_parnum);
+      memset(d_req_bind, 0, sizeof(MYSQL_BIND) * d_parnum);
     }
 
     d_prepared = true;
   }
 
-  void releaseStatement() {
+  void releaseStatement()
+  {
     d_prepared = false;
     if (d_stmt)
       mysql_stmt_close(d_stmt);
-    d_stmt = NULL;
+    d_stmt = nullptr;
     if (d_req_bind) {
-      for(int i=0;i<d_parnum;i++) {
-        if (d_req_bind[i].buffer) delete [] (char*)d_req_bind[i].buffer;
-        if (d_req_bind[i].length) delete [] d_req_bind[i].length;
+      for (int i = 0; i < d_parnum; i++) {
+        if (d_req_bind[i].buffer)
+          delete[](char*) d_req_bind[i].buffer;
+        if (d_req_bind[i].length)
+          delete[] d_req_bind[i].length;
       }
-      delete [] d_req_bind;
-      d_req_bind = NULL;
+      delete[] d_req_bind;
+      d_req_bind = nullptr;
     }
     if (d_res_bind) {
-      for(int i=0;i<d_fnum;i++) {
-        if (d_res_bind[i].buffer) delete [] (char*)d_res_bind[i].buffer;
-        if (d_res_bind[i].length) delete [] d_res_bind[i].length;
-        if (d_res_bind[i].error) delete [] d_res_bind[i].error;
-        if (d_res_bind[i].is_null) delete [] d_res_bind[i].is_null;
+      for (int i = 0; i < d_fnum; i++) {
+        if (d_res_bind[i].buffer)
+          delete[](char*) d_res_bind[i].buffer;
+        if (d_res_bind[i].length)
+          delete[] d_res_bind[i].length;
+        if (d_res_bind[i].error)
+          delete[] d_res_bind[i].error;
+        if (d_res_bind[i].is_null)
+          delete[] d_res_bind[i].is_null;
       }
-      delete [] d_res_bind;
-      d_res_bind = NULL;
+      delete[] d_res_bind;
+      d_res_bind = nullptr;
     }
     d_paridx = d_fnum = d_resnum = d_residx = 0;
   }
@@ -428,7 +461,7 @@ private:
   MYSQL_BIND* d_res_bind;
 
   string d_query;
-  
+
   bool d_prepared;
   bool d_dolog;
   DTime d_dtime; // only used if d_dolog is set
@@ -441,7 +474,7 @@ private:
 
 void SMySQL::connect()
 {
-  int retry=1;
+  int retry = 1;
 
   std::lock_guard<std::mutex> l(s_myinitlock);
   if (d_threadCleanup) {
@@ -459,48 +492,49 @@ void SMySQL::connect()
 #endif
 
 #if MYSQL_VERSION_ID >= 50100
-    if(d_timeout) {
+    if (d_timeout) {
       mysql_options(&d_db, MYSQL_OPT_READ_TIMEOUT, &d_timeout);
       mysql_options(&d_db, MYSQL_OPT_WRITE_TIMEOUT, &d_timeout);
     }
 #endif
 
     if (d_setIsolation && (retry == 1))
-      mysql_options(&d_db, MYSQL_INIT_COMMAND,"SET SESSION TRANSACTION ISOLATION LEVEL READ COMMITTED");
+      mysql_options(&d_db, MYSQL_INIT_COMMAND, "SET SESSION TRANSACTION ISOLATION LEVEL READ COMMITTED");
 
     mysql_options(&d_db, MYSQL_READ_DEFAULT_GROUP, d_group.c_str());
 
-    if (!mysql_real_connect(&d_db, d_host.empty() ? NULL : d_host.c_str(),
-                            d_user.empty() ? NULL : d_user.c_str(),
-                            d_password.empty() ? NULL : d_password.c_str(),
-                            d_database.empty() ? NULL : d_database.c_str(),
+    if (!mysql_real_connect(&d_db, d_host.empty() ? nullptr : d_host.c_str(),
+                            d_user.empty() ? nullptr : d_user.c_str(),
+                            d_password.empty() ? nullptr : d_password.c_str(),
+                            d_database.empty() ? nullptr : d_database.c_str(),
                             d_port,
-                            d_msocket.empty() ? NULL : d_msocket.c_str(),
+                            d_msocket.empty() ? nullptr : d_msocket.c_str(),
                             (d_clientSSL ? CLIENT_SSL : 0) | CLIENT_MULTI_RESULTS)) {
 
       if (retry == 0)
         throw sPerrorException("Unable to connect to database");
       --retry;
-    } else {
+    }
+    else {
       if (retry == 0) {
         mysql_close(&d_db);
         throw sPerrorException("Please add '(gmysql-)innodb-read-committed=no' to your PowerDNS configuration, and reconsider your storage engine if it does not support transactions.");
       }
-      retry=-1;
+      retry = -1;
     }
   } while (retry >= 0);
 }
 
-SMySQL::SMySQL(const string &database, const string &host, uint16_t port, const string &msocket, const string &user,
-               const string &password, const string &group, bool setIsolation, unsigned int timeout, bool threadCleanup, bool clientSSL):
-  d_database(database), d_host(host), d_msocket(msocket), d_user(user), d_password(password), d_group(group), d_timeout(timeout), d_port(port), d_setIsolation(setIsolation), d_threadCleanup(threadCleanup), d_clientSSL(clientSSL)
+SMySQL::SMySQL(string database, string host, uint16_t port, string msocket, string user,
+               string password, string group, bool setIsolation, unsigned int timeout, bool threadCleanup, bool clientSSL) :
+  d_database(std::move(database)), d_host(std::move(host)), d_msocket(std::move(msocket)), d_user(std::move(user)), d_password(std::move(password)), d_group(std::move(group)), d_timeout(timeout), d_port(port), d_setIsolation(setIsolation), d_threadCleanup(threadCleanup), d_clientSSL(clientSSL)
 {
   connect();
 }
 
 void SMySQL::setLog(bool state)
 {
-  s_dolog=state;
+  s_dolog = state;
 }
 
 SMySQL::~SMySQL()
@@ -508,9 +542,9 @@ SMySQL::~SMySQL()
   mysql_close(&d_db);
 }
 
-SSqlException SMySQL::sPerrorException(const string &reason)
+SSqlException SMySQL::sPerrorException(const string& reason)
 {
-  return SSqlException(reason+string(": ERROR ")+std::to_string(mysql_errno(&d_db))+" ("+string(mysql_sqlstate(&d_db))+"): "+mysql_error(&d_db));
+  return SSqlException(reason + string(": ERROR ") + std::to_string(mysql_errno(&d_db)) + " (" + string(mysql_sqlstate(&d_db)) + "): " + mysql_error(&d_db));
 }
 
 std::unique_ptr<SSqlStatement> SMySQL::prepare(const string& query, int nparams)
@@ -520,23 +554,26 @@ std::unique_ptr<SSqlStatement> SMySQL::prepare(const string& query, int nparams)
 
 void SMySQL::execute(const string& query)
 {
-  if(s_dolog)
-    g_log<<Logger::Warning<<"Query: "<<query<<endl;
+  if (s_dolog)
+    g_log << Logger::Warning << "Query: " << query << endl;
 
   int err;
-  if((err=mysql_query(&d_db,query.c_str())))
-    throw sPerrorException("Failed to execute mysql_query '" + query + "' Err="+itoa(err));
+  if ((err = mysql_query(&d_db, query.c_str())))
+    throw sPerrorException("Failed to execute mysql_query '" + query + "' Err=" + itoa(err));
 }
 
-void SMySQL::startTransaction() {
+void SMySQL::startTransaction()
+{
   execute("begin");
 }
 
-void SMySQL::commit() {
+void SMySQL::commit()
+{
   execute("commit");
 }
 
-void SMySQL::rollback() {
+void SMySQL::rollback()
+{
   execute("rollback");
 }
 
