@@ -34,11 +34,13 @@
 #include "pdns/statbag.hh"
 #include "pdns/auth-packetcache.hh"
 #include "pdns/auth-querycache.hh"
+#include "pdns/auth-zonecache.hh"
 
 StatBag S;
 AuthPacketCache PC;
 AuthQueryCache QC;
-ArgvMap &arg()
+AuthZoneCache g_zoneCache;
+ArgvMap& arg()
 {
   static ArgvMap arg;
   return arg;
@@ -46,11 +48,11 @@ ArgvMap &arg()
 
 class RemoteLoader
 {
-   public:
-      RemoteLoader();
+public:
+  RemoteLoader();
 };
 
-DNSBackend *be;
+DNSBackend* be;
 
 #define BOOST_TEST_DYN_LINK
 #define BOOST_TEST_MAIN
@@ -61,23 +63,26 @@ DNSBackend *be;
 
 #include <boost/tuple/tuple.hpp>
 
-struct RemotebackendSetup {
-    RemotebackendSetup()  {
-	be = 0; 
-	try {
-		// setup minimum arguments
-		::arg().set("module-dir")="./.libs";
-                new RemoteLoader();
-		BackendMakers().launch("remote");
-                // then get us a instance of it 
-                ::arg().set("remote-connection-string")="http:url=http://localhost:62434/dns";
-                ::arg().set("remote-dnssec")="yes";
-                be = BackendMakers().all()[0];
-	} catch (PDNSException &ex) {
-		BOOST_TEST_MESSAGE("Cannot start remotebackend: " << ex.reason );
-	};
+struct RemotebackendSetup
+{
+  RemotebackendSetup()
+  {
+    be = 0;
+    try {
+      // setup minimum arguments
+      ::arg().set("module-dir") = "./.libs";
+      new RemoteLoader();
+      BackendMakers().launch("remote");
+      // then get us a instance of it
+      ::arg().set("remote-connection-string") = "http:url=http://localhost:62434/dns";
+      ::arg().set("remote-dnssec") = "yes";
+      be = BackendMakers().all()[0];
     }
-    ~RemotebackendSetup()  {  }
+    catch (PDNSException& ex) {
+      BOOST_TEST_MESSAGE("Cannot start remotebackend: " << ex.reason);
+    };
+  }
+  ~RemotebackendSetup() {}
 };
 
-BOOST_GLOBAL_FIXTURE( RemotebackendSetup );
+BOOST_GLOBAL_FIXTURE(RemotebackendSetup);
