@@ -27,7 +27,7 @@
 #include "pdns/dnsrecords.hh"
 #include <utility>
 
-static string backendname="[TinyDNSBackend] ";
+static string backendname = "[TinyDNSBackend] ";
 uint32_t TinyDNSBackend::s_lastId;
 std::mutex TinyDNSBackend::s_domainInfoLock;
 TinyDNSBackend::TDI_suffix_t TinyDNSBackend::s_domainInfo;
@@ -36,7 +36,7 @@ vector<string> TinyDNSBackend::getLocations()
 {
   vector<string> ret;
 
-  if (! d_dnspacket) {
+  if (!d_dnspacket) {
     return ret;
   }
 
@@ -51,19 +51,19 @@ vector<string> TinyDNSBackend::getLocations()
   char key[6];
   key[0] = '\000';
   key[1] = '\045';
-  key[2] = (addr      )&0xff;
-  key[3] = (addr >>  8)&0xff;
-  key[4] = (addr >> 16)&0xff;
-  key[5] = (addr >> 24)&0xff;
+  key[2] = (addr)&0xff;
+  key[3] = (addr >> 8) & 0xff;
+  key[4] = (addr >> 16) & 0xff;
+  key[5] = (addr >> 24) & 0xff;
 
-  for (int i=4;i>=0;i--) {
-    string searchkey(key, i+2);
+  for (int i = 4; i >= 0; i--) {
+    string searchkey(key, i + 2);
     try {
       auto reader = std::unique_ptr<CDB>(new CDB(getArg("dbfile")));
       ret = reader->findall(searchkey);
     }
-    catch(const std::exception& e) {
-      g_log<<Logger::Error<<e.what()<<endl;
+    catch (const std::exception& e) {
+      g_log << Logger::Error << e.what() << endl;
       throw PDNSException(e.what());
     }
 
@@ -76,9 +76,9 @@ vector<string> TinyDNSBackend::getLocations()
   return ret;
 }
 
-TinyDNSBackend::TinyDNSBackend(const string &suffix)
+TinyDNSBackend::TinyDNSBackend(const string& suffix)
 {
-  setArgPrefix("tinydns"+suffix);
+  setArgPrefix("tinydns" + suffix);
   d_suffix = suffix;
   d_locations = mustDo("locations");
   d_ignorebogus = mustDo("ignore-bogus-records");
@@ -89,25 +89,26 @@ TinyDNSBackend::TinyDNSBackend(const string &suffix)
   d_isWildcardQuery = false;
 }
 
-void TinyDNSBackend::getUpdatedMasters(vector<DomainInfo>* retDomains) {
+void TinyDNSBackend::getUpdatedMasters(vector<DomainInfo>* retDomains)
+{
   std::lock_guard<std::mutex> l(s_domainInfoLock); //TODO: We could actually lock less if we do it per suffix.
 
-  if (! s_domainInfo.count(d_suffix)) {
+  if (!s_domainInfo.count(d_suffix)) {
     TDI_t tmp;
-    s_domainInfo.insert( make_pair(d_suffix,tmp) );
+    s_domainInfo.insert(make_pair(d_suffix, tmp));
   }
 
-  TDI_t *domains = &s_domainInfo[d_suffix];
+  TDI_t* domains = &s_domainInfo[d_suffix];
 
   vector<DomainInfo> allDomains;
   getAllDomains(&allDomains);
   if (domains->size() == 0 && !mustDo("notify-on-startup")) {
-    for (vector<DomainInfo>::iterator di=allDomains.begin(); di!=allDomains.end(); ++di) {
+    for (vector<DomainInfo>::iterator di = allDomains.begin(); di != allDomains.end(); ++di) {
       di->notified_serial = 0;
     }
   }
 
-  for(vector<DomainInfo>::iterator di=allDomains.begin(); di!=allDomains.end(); ++di) {
+  for (vector<DomainInfo>::iterator di = allDomains.begin(); di != allDomains.end(); ++di) {
     TDIByZone_t& zone_index = domains->get<tag_zone>();
     TDIByZone_t::iterator itByZone = zone_index.find(di->zone);
     if (itByZone == zone_index.end()) {
@@ -123,7 +124,8 @@ void TinyDNSBackend::getUpdatedMasters(vector<DomainInfo>* retDomains) {
       if (di->notified_serial > 0) {
         retDomains->push_back(*di);
       }
-    } else {
+    }
+    else {
       if (itByZone->notified_serial < di->serial) {
         di->id = itByZone->id;
         retDomains->push_back(*di);
@@ -132,32 +134,36 @@ void TinyDNSBackend::getUpdatedMasters(vector<DomainInfo>* retDomains) {
   }
 }
 
-void TinyDNSBackend::setNotified(uint32_t id, uint32_t serial) {
+void TinyDNSBackend::setNotified(uint32_t id, uint32_t serial)
+{
   std::lock_guard<std::mutex> l(s_domainInfoLock);
   if (!s_domainInfo.count(d_suffix)) {
     throw PDNSException("Can't get list of domains to set the serial.");
   }
-  TDI_t *domains = &s_domainInfo[d_suffix];
+  TDI_t* domains = &s_domainInfo[d_suffix];
   TDIById_t& domain_index = domains->get<tag_domainid>();
   TDIById_t::iterator itById = domain_index.find(id);
   if (itById == domain_index.end()) {
-    g_log<<Logger::Error<<backendname<<"Received updated serial("<<serial<<"), but domain ID ("<<id<<") is not known in this backend."<<endl;
-  } else {
-    DLOG(g_log<<Logger::Debug<<backendname<<"Setting serial for "<<itById->zone<<" to "<<serial<<endl);
+    g_log << Logger::Error << backendname << "Received updated serial(" << serial << "), but domain ID (" << id << ") is not known in this backend." << endl;
+  }
+  else {
+    DLOG(g_log << Logger::Debug << backendname << "Setting serial for " << itById->zone << " to " << serial << endl);
     domain_index.modify(itById, TDI_SerialModifier(serial));
   }
   s_domainInfo[d_suffix] = *domains;
 }
 
-void TinyDNSBackend::getAllDomains(vector<DomainInfo> *domains, bool include_disabled) {
-  d_isAxfr=true;
+void TinyDNSBackend::getAllDomains(vector<DomainInfo>* domains, bool include_disabled)
+{
+  d_isAxfr = true;
+  d_isGetDomains = true;
   d_dnspacket = NULL;
 
   try {
-    d_cdbReader=std::unique_ptr<CDB>(new CDB(getArg("dbfile")));
+    d_cdbReader = std::unique_ptr<CDB>(new CDB(getArg("dbfile")));
   }
   catch (const std::exception& e) {
-    g_log<<Logger::Error<<e.what()<<endl;
+    g_log << Logger::Error << e.what() << endl;
     throw PDNSException(e.what());
   }
 
@@ -167,58 +173,68 @@ void TinyDNSBackend::getAllDomains(vector<DomainInfo> *domains, bool include_dis
 
   while (get(rr)) {
     if (rr.qtype.getCode() == QType::SOA && dupcheck.insert(rr.qname).second) {
-      SOAData sd;
-      fillSOAData(rr.content, sd);
-
       DomainInfo di;
       di.id = -1; //TODO: Check if this is ok.
-      di.backend=this;
+      di.backend = this;
       di.zone = rr.qname;
-      di.serial = sd.serial;
-      di.notified_serial = sd.serial;
       di.kind = DomainInfo::Master;
       di.last_check = time(0);
+
+      SOAData sd;
+      try {
+        fillSOAData(rr.content, sd);
+        di.serial = sd.serial;
+      }
+      catch (...) {
+        di.serial = 0;
+      }
+
+      di.notified_serial = di.serial;
       domains->push_back(di);
     }
   }
 }
 
-bool TinyDNSBackend::list(const DNSName &target, int domain_id, bool include_disabled) {
-  d_isAxfr=true;
+bool TinyDNSBackend::list(const DNSName& target, int domain_id, bool include_disabled)
+{
+  d_isAxfr = true;
+  d_isGetDomains = false;
   string key = target.toDNSStringLC();
   try {
-    d_cdbReader=std::unique_ptr<CDB>(new CDB(getArg("dbfile")));
+    d_cdbReader = std::unique_ptr<CDB>(new CDB(getArg("dbfile")));
   }
   catch (const std::exception& e) {
-    g_log<<Logger::Error<<e.what()<<endl;
+    g_log << Logger::Error << e.what() << endl;
     throw PDNSException(e.what());
   }
 
   return d_cdbReader->searchSuffix(key);
 }
 
-void TinyDNSBackend::lookup(const QType &qtype, const DNSName &qdomain, int zoneId, DNSPacket *pkt_p) {
+void TinyDNSBackend::lookup(const QType& qtype, const DNSName& qdomain, int zoneId, DNSPacket* pkt_p)
+{
   d_isAxfr = false;
+  d_isGetDomains = false;
   string queryDomain = toLowerCanonic(qdomain.toString());
 
-  string key=simpleCompress(queryDomain);
+  string key = simpleCompress(queryDomain);
 
-  DLOG(g_log<<Logger::Debug<<backendname<<"[lookup] query for qtype ["<<qtype.getName()<<"] qdomain ["<<qdomain<<"]"<<endl);
-  DLOG(g_log<<Logger::Debug<<"[lookup] key ["<<makeHexDump(key)<<"]"<<endl);
+  DLOG(g_log << Logger::Debug << backendname << "[lookup] query for qtype [" << qtype.toString() << "] qdomain [" << qdomain << "]" << endl);
+  DLOG(g_log << Logger::Debug << "[lookup] key [" << makeHexDump(key) << "]" << endl);
 
   d_isWildcardQuery = false;
   if (key[0] == '\001' && key[1] == '\052') {
     d_isWildcardQuery = true;
-    key.erase(0,2);
+    key.erase(0, 2);
   }
 
-  d_qtype=qtype;
+  d_qtype = qtype;
 
   try {
-    d_cdbReader=std::unique_ptr<CDB>(new CDB(getArg("dbfile")));
+    d_cdbReader = std::unique_ptr<CDB>(new CDB(getArg("dbfile")));
   }
   catch (const std::exception& e) {
-    g_log<<Logger::Error<<e.what()<<endl;
+    g_log << Logger::Error << e.what() << endl;
     throw PDNSException(e.what());
   }
 
@@ -226,8 +242,7 @@ void TinyDNSBackend::lookup(const QType &qtype, const DNSName &qdomain, int zone
   d_dnspacket = pkt_p;
 }
 
-
-bool TinyDNSBackend::get(DNSResourceRecord &rr)
+bool TinyDNSBackend::get(DNSResourceRecord& rr)
 {
   pair<string, string> record;
 
@@ -253,13 +268,16 @@ bool TinyDNSBackend::get(DNSResourceRecord &rr)
       }
     }
 
-
     PacketReader pr(val, 0);
     rr.qtype = QType(pr.get16BitInt());
 
-    if(d_isAxfr || d_qtype.getCode() == QType::ANY || rr.qtype == d_qtype) {
+    if (d_isGetDomains && rr.qtype != QType::SOA) {
+      continue;
+    }
+
+    if (d_isAxfr || d_qtype.getCode() == QType::ANY || rr.qtype == d_qtype) {
       char locwild = pr.get8BitInt();
-      if(locwild != '\075' && (locwild == '\076' || locwild == '\053')) {
+      if (locwild != '\075' && (locwild == '\076' || locwild == '\053')) {
         if (d_isAxfr && d_locations) { // We skip records with a location in AXFR, unless we disable locations.
           continue;
         }
@@ -270,7 +288,7 @@ bool TinyDNSBackend::get(DNSResourceRecord &rr)
         if (d_locations) {
           bool foundLocation = false;
           vector<string> locations = getLocations();
-          while(locations.size() > 0) {
+          while (locations.size() > 0) {
             string locId = locations.back();
             locations.pop_back();
 
@@ -285,13 +303,13 @@ bool TinyDNSBackend::get(DNSResourceRecord &rr)
         }
       }
 
-      if (d_isAxfr && (val[2] == '\052' || val[2] == '\053' )) { // Keys are not stored with wildcard character, with AXFR we need to add that.
+      if (d_isAxfr && (val[2] == '\052' || val[2] == '\053')) { // Keys are not stored with wildcard character, with AXFR we need to add that.
         key.insert(0, 1, '\052');
         key.insert(0, 1, '\001');
       }
       // rr.qname.clear();
-      rr.qname=DNSName(key.c_str(), key.size(), 0, false);
-      rr.domain_id=-1;
+      rr.qname = DNSName(key.c_str(), key.size(), 0, false);
+      rr.domain_id = -1;
       // 11:13.21 <@ahu> IT IS ALWAYS AUTH --- well not really because we are just a backend :-)
       // We could actually do NSEC3-NARROW DNSSEC according to Habbie, if we do, we need to change something here.
       rr.auth = true;
@@ -300,16 +318,19 @@ bool TinyDNSBackend::get(DNSResourceRecord &rr)
       uint64_t timestamp = pr.get32BitInt();
       timestamp <<= 32;
       timestamp += pr.get32BitInt();
-      if(timestamp) {
+      if (timestamp) {
         uint64_t now = d_taiepoch + time(NULL);
         if (rr.ttl == 0) {
           if (timestamp < now) {
             continue;
           }
           rr.ttl = timestamp - now;
-          if (rr.ttl <= 2) rr.ttl = 2;
-          if (rr.ttl >= 3600) rr.ttl = 3600;
-        } else if (now <= timestamp) {
+          if (rr.ttl <= 2)
+            rr.ttl = 2;
+          if (rr.ttl >= 3600)
+            rr.ttl = 3600;
+        }
+        else if (now <= timestamp) {
           continue;
         }
       }
@@ -317,39 +338,42 @@ bool TinyDNSBackend::get(DNSResourceRecord &rr)
         DNSRecord dr;
         dr.d_class = 1;
         dr.d_type = rr.qtype.getCode();
-        dr.d_clen = val.size()-pr.getPosition();
+        dr.d_clen = val.size() - pr.getPosition();
 
         auto drc = DNSRecordContent::mastermake(dr, pr);
         rr.content = drc->getZoneRepresentation();
-        DLOG(cerr<<"CONTENT: "<<rr.content<<endl);
+        DLOG(cerr << "CONTENT: " << rr.content << endl);
       }
       catch (...) {
-        g_log<<Logger::Error<<backendname<<"Failed to parse record content for "<<rr.qname<<" with type "<<rr.qtype.getName();
-        if (d_ignorebogus) {
-          g_log<<". Ignoring!"<<endl;
+        g_log << Logger::Error << backendname << "Failed to parse record content for " << rr.qname << " with type " << rr.qtype.toString();
+        if (d_ignorebogus || d_isGetDomains) {
+          g_log << ". Ignoring!" << endl;
           continue;
-        } else {
-          g_log<<". Erroring out!"<<endl;
+        }
+        else {
+          g_log << ". Erroring out!" << endl;
           throw;
         }
       }
-//      DLOG(g_log<<Logger::Debug<<backendname<<"Returning ["<<rr.content<<"] for ["<<rr.qname<<"] of RecordType ["<<rr.qtype.getName()<<"]"<<endl;);
+      //      DLOG(g_log<<Logger::Debug<<backendname<<"Returning ["<<rr.content<<"] for ["<<rr.qname<<"] of RecordType ["<<rr.qtype.toString()<<"]"<<endl;);
       return true;
     }
   } // end of while
-  DLOG(g_log<<Logger::Debug<<backendname<<"No more records to return."<<endl);
+  DLOG(g_log << Logger::Debug << backendname << "No more records to return." << endl);
 
   d_cdbReader = nullptr;
   return false;
 }
 
 // boilerplate
-class TinyDNSFactory: public BackendFactory
+class TinyDNSFactory : public BackendFactory
 {
 public:
-  TinyDNSFactory() : BackendFactory("tinydns") {}
+  TinyDNSFactory() :
+    BackendFactory("tinydns") {}
 
-  void declareArguments(const string &suffix="") override {
+  void declareArguments(const string& suffix = "") override
+  {
     declare(suffix, "notify-on-startup", "Tell the TinyDNSBackend to notify all the slave nameservers on startup. Default is no.", "no");
     declare(suffix, "dbfile", "Location of the cdb data file", "data.cdb");
     declare(suffix, "tai-adjust", "This adjusts the TAI value if timestamps are used. These seconds will be added to the start point (1970) and will allow you to adjust for leap seconds. The default is 11.", "11");
@@ -357,7 +381,8 @@ public:
     declare(suffix, "ignore-bogus-records", "The data.cdb file might have some incorrect record data, this causes PowerDNS to fail, where tinydns would send out truncated data. This option makes powerdns ignore that data!", "no");
   }
 
-  DNSBackend *make(const string &suffix="") override {
+  DNSBackend* make(const string& suffix = "") override
+  {
     return new TinyDNSBackend(suffix);
   }
 };
@@ -366,13 +391,14 @@ public:
 class TinyDNSLoader
 {
 public:
-  TinyDNSLoader() {
+  TinyDNSLoader()
+  {
     BackendMakers().report(new TinyDNSFactory);
     g_log << Logger::Info << "[tinydnsbackend] This is the tinydns backend version " VERSION
 #ifndef REPRODUCIBLE
-      << " (" __DATE__ " " __TIME__ ")"
+          << " (" __DATE__ " " __TIME__ ")"
 #endif
-      << " reporting" << endl;
+          << " reporting" << endl;
   }
 };
 
