@@ -132,8 +132,8 @@ void DNSPacket::addRecord(DNSZoneRecord&& rr)
     std::string ser = const_cast<DNSZoneRecord&>(rr).dr.d_content->serialize(rr.dr.d_name);
     auto hash = boost::hash< std::pair<DNSName, std::string> >()({rr.dr.d_name, ser});
     if(d_dedup.count(hash)) { // might be a dup
-      for(auto i=d_rrs.begin();i!=d_rrs.end();++i) {
-        if(rr.dr == i->dr)  // XXX SUPER SLOW
+      for(auto & i : d_rrs) {
+        if(rr.dr == i.dr)  // XXX SUPER SLOW
           return;
       }
     }
@@ -147,31 +147,40 @@ vector<DNSZoneRecord*> DNSPacket::getAPRecords()
 {
   vector<DNSZoneRecord*> arrs;
 
-  for(vector<DNSZoneRecord>::iterator i=d_rrs.begin();
-      i!=d_rrs.end();
-      ++i)
+  for(auto & i : d_rrs)
     {
-      if(i->dr.d_place!=DNSResourceRecord::ADDITIONAL &&
-         (i->dr.d_type==QType::MX ||
-          i->dr.d_type==QType::NS ||
-          i->dr.d_type==QType::SRV))
+      if(i.dr.d_place!=DNSResourceRecord::ADDITIONAL &&
+         (i.dr.d_type==QType::MX ||
+          i.dr.d_type==QType::NS ||
+          i.dr.d_type==QType::SRV))
         {
-          arrs.push_back(&*i);
+          arrs.push_back(&i);
         }
     }
   return arrs;
 }
 
+vector<DNSZoneRecord*> DNSPacket::getServiceRecords()
+{
+  vector<DNSZoneRecord*> arrs;
+
+  for(auto & i : d_rrs) {
+    if (i.dr.d_type==QType::SVCB ||
+        i.dr.d_type==QType::HTTPS) {
+      arrs.push_back(&i);
+    }
+  }
+  return arrs;
+}
+
 vector<DNSZoneRecord*> DNSPacket::getAnswerRecords()
 {
   vector<DNSZoneRecord*> arrs;
 
-  for(vector<DNSZoneRecord>::iterator i=d_rrs.begin();
-      i!=d_rrs.end();
-      ++i)
+  for(auto & rr : d_rrs)
     {
-      if(i->dr.d_place!=DNSResourceRecord::ADDITIONAL)
-        arrs.push_back(&*i);
+      if(rr.dr.d_place!=DNSResourceRecord::ADDITIONAL)
+        arrs.push_back(&rr);
     }
   return arrs;
 }
@@ -362,7 +371,7 @@ std::unique_ptr<DNSPacket> DNSPacket::replyPacket() const
   r->setRemote(&d_remote);
   r->setAnswer(true);  // this implies the allocation of the header
   r->setA(true); // and we are authoritative
-  r->setRA(0); // no recursion available
+  r->setRA(false); // no recursion available
   r->setRD(d.rd); // if you wanted to recurse, answer will say you wanted it 
   r->setID(d.id);
   r->setOpcode(d.opcode);
@@ -439,18 +448,18 @@ bool DNSPacket::getTSIGDetails(TSIGRecordContent* trc, DNSName* keyname, uint16_
   uint16_t tsigPos = mdp.getTSIGPos();
   if(!tsigPos)
     return false;
-  
+
   bool gotit=false;
-  for(MOADNSParser::answers_t::const_iterator i=mdp.d_answers.begin(); i!=mdp.d_answers.end(); ++i) {          
-    if(i->first.d_type == QType::TSIG && i->first.d_class == QType::ANY) {
+  for(const auto & answer : mdp.d_answers) {
+    if(answer.first.d_type == QType::TSIG && answer.first.d_class == QType::ANY) {
       // cast can fail, f.e. if d_content is an UnknownRecordContent.
-      shared_ptr<TSIGRecordContent> content = std::dynamic_pointer_cast<TSIGRecordContent>(i->first.d_content);
+      shared_ptr<TSIGRecordContent> content = std::dynamic_pointer_cast<TSIGRecordContent>(answer.first.d_content);
       if (!content) {
         g_log<<Logger::Error<<"TSIG record has no or invalid content (invalid packet)"<<endl;
         return false;
       }
       *trc = *content;
-      *keyname = i->first.d_name;
+      *keyname = answer.first.d_name;
       gotit=true;
     }
   }
@@ -460,7 +469,7 @@ bool DNSPacket::getTSIGDetails(TSIGRecordContent* trc, DNSName* keyname, uint16_
   if (tsigPosOut) {
     *tsigPosOut = tsigPos;
   }
-  
+
   return true;
 }
 
@@ -469,21 +478,21 @@ bool DNSPacket::getTKEYRecord(TKEYRecordContent *tr, DNSName *keyname) const
   MOADNSParser mdp(d_isQuery, d_rawpacket);
   bool gotit=false;
 
-  for(MOADNSParser::answers_t::const_iterator i=mdp.d_answers.begin(); i!=mdp.d_answers.end(); ++i) {
+  for(const auto & answer : mdp.d_answers) {
     if (gotit) {
       g_log<<Logger::Error<<"More than one TKEY record found in query"<<endl;
       return false;
     }
 
-    if(i->first.d_type == QType::TKEY) {
+    if(answer.first.d_type == QType::TKEY) {
       // cast can fail, f.e. if d_content is an UnknownRecordContent.
-      shared_ptr<TKEYRecordContent> content = std::dynamic_pointer_cast<TKEYRecordContent>(i->first.d_content);
+      shared_ptr<TKEYRecordContent> content = std::dynamic_pointer_cast<TKEYRecordContent>(answer.first.d_content);
       if (!content) {
         g_log<<Logger::Error<<"TKEY record has no or invalid content (invalid packet)"<<endl;
         return false;
       }
       *tr = *content;
-      *keyname = i->first.d_name;
+      *keyname = answer.first.d_name;
       gotit=true;
     }
   }
@@ -528,14 +537,12 @@ try
     if(edo.d_extFlags & EDNSOpts::DNSSECOK)
       d_dnssecOk=true;
 
-    for(vector<pair<uint16_t, string> >::const_iterator iter = edo.d_options.begin();
-        iter != edo.d_options.end(); 
-        ++iter) {
-      if(iter->first == EDNSOptionCode::NSID) {
+    for(const auto & option : edo.d_options) {
+      if(option.first == EDNSOptionCode::NSID) {
         d_wantsnsid=true;
       }
-      else if(s_doEDNSSubnetProcessing && (iter->first == EDNSOptionCode::ECS)) { // 'EDNS SUBNET'
-        if(getEDNSSubnetOptsFromString(iter->second, &d_eso)) {
+      else if(s_doEDNSSubnetProcessing && (option.first == EDNSOptionCode::ECS)) { // 'EDNS SUBNET'
+        if(getEDNSSubnetOptsFromString(option.second, &d_eso)) {
           //cerr<<"Parsed, source: "<<d_eso.source.toString()<<", scope: "<<d_eso.scope.toString()<<", family = "<<d_eso.scope.getNetwork().sin4.sin_family<<endl;
           d_haveednssubnet=true;
         } 
