@@ -2,6 +2,7 @@
 #include <future>
 #include <mutex>
 #include <boost/format.hpp>
+#include <utility>
 #include "version.hh"
 #include "ext/luawrapper/include/LuaContext.hpp"
 #include "lua-auth4.hh"
@@ -24,7 +25,7 @@
 
    ponder netmask tree from file for huge number of netmasks
 
-   unify ifupurl/ifupport
+   unify ifurlup/ifportup
       add attribute for certificate check
    add list of current monitors
       expire them too?
@@ -88,7 +89,7 @@ private:
   void checkURL(const CheckDesc& cd, const bool status, const bool first = false)
   {
     try {
-      int timeout = 1;
+      int timeout = 2;
       if (cd.opts.count("timeout")) {
         timeout = std::atoi(cd.opts.at("timeout").c_str());
       }
@@ -122,7 +123,7 @@ private:
   }
   void checkTCP(const CheckDesc& cd, const bool status, const bool first = false) {
     try {
-      int timeout = 1;
+      int timeout = 2;
       if (cd.opts.count("timeout")) {
         timeout = std::atoi(cd.opts.at("timeout").c_str());
       }
@@ -488,6 +489,7 @@ static vector<string> convIpListToString(const vector<ComboAddress> &comboAddres
 {
   vector<string> ret;
 
+  ret.reserve(comboAddresses.size());
   for (const auto& c : comboAddresses) {
     ret.emplace_back(c.toString());
   }
@@ -506,10 +508,11 @@ static vector<ComboAddress> convIplist(const iplist_t& src)
   return ret;
 }
 
-static vector<pair<int, ComboAddress> > convWIplist(std::unordered_map<int, wiplist_t > src)
+static vector<pair<int, ComboAddress> > convWIplist(const std::unordered_map<int, wiplist_t >& src)
 {
   vector<pair<int,ComboAddress> > ret;
 
+  ret.reserve(src.size());
   for(const auto& i : src) {
     ret.emplace_back(atoi(i.second.at(1).c_str()), ComboAddress(i.second.at(2)));
   }
@@ -568,7 +571,7 @@ static void setupLuaRecords()
     });
 
 
-  lua.writeFunction("createReverse", [](string suffix, boost::optional<std::unordered_map<string,string>> e){
+  lua.writeFunction("createReverse", [](string format, boost::optional<std::unordered_map<string,string>> e){
       try {
         auto labels = s_lua_record_ctx->qname.getRawLabels();
         if(labels.size()<4)
@@ -576,10 +579,8 @@ static void setupLuaRecords()
         
         vector<ComboAddress> candidates;
         
-        // exceptions are relative to zone
         // so, query comes in for 4.3.2.1.in-addr.arpa, zone is called 2.1.in-addr.arpa
-        // e["1.2.3.4"]="bert.powerdns.com" - should match, easy enough to do
-        // the issue is with classless delegation..
+        // e["1.2.3.4"]="bert.powerdns.com" then provides an exception
         if(e) {
           ComboAddress req(labels[3]+"."+labels[2]+"."+labels[1]+"."+labels[0], 0);
           const auto& uom = *e;
@@ -587,7 +588,7 @@ static void setupLuaRecords()
             if(ComboAddress(c.first, 0) == req)
               return c.second;
         }
-        boost::format fmt(suffix);
+        boost::format fmt(format);
         fmt.exceptions( boost::io::all_error_bits ^ ( boost::io::too_many_args_bit | boost::io::too_few_args_bit )  );
         fmt % labels[3] % labels[2] % labels[1] % labels[0];
         
@@ -671,14 +672,14 @@ static void setupLuaRecords()
 
       return std::string("::");
     });
-  lua.writeFunction("createReverse6", [](string suffix, boost::optional<std::unordered_map<string,string>> e){
+  lua.writeFunction("createReverse6", [](string format, boost::optional<std::unordered_map<string,string>> e){
       vector<ComboAddress> candidates;
 
       try {
         auto labels= s_lua_record_ctx->qname.getRawLabels();
         if(labels.size()<32)
           return std::string("unknown");
-        boost::format fmt(suffix);
+        boost::format fmt(format);
         fmt.exceptions( boost::io::all_error_bits ^ ( boost::io::too_many_args_bit | boost::io::too_few_args_bit )  );
 
 
@@ -718,7 +719,7 @@ static void setupLuaRecords()
         return fmt.str();
       }
       catch(std::exception& ex) {
-        g_log<<Logger::Error<<"LUA Record xception: "<<ex.what()<<endl;
+        g_log<<Logger::Error<<"LUA Record exception: "<<ex.what()<<endl;
       }
       catch(PDNSException& ex) {
         g_log<<Logger::Error<<"LUA Record exception: "<<ex.reason<<endl;
@@ -726,6 +727,24 @@ static void setupLuaRecords()
       return std::string("unknown");
     });
 
+  lua.writeFunction("filterForward", [](string address, NetmaskGroup& nmg, boost::optional<string> fallback) {
+      ComboAddress ca(address);
+
+      if (nmg.match(ComboAddress(address))) {
+        return address;
+      } else {
+        if (fallback) {
+          return *fallback;
+        }
+
+        if (ca.isIPv4()) {
+          return string("0.0.0.0");
+        } else {
+          return string("::");
+        }
+      }
+    });
+
   /*
    * Simplistic test to see if an IP address listens on a certain port
    * Will return a single IP address from the set of available IP addresses. If
@@ -834,6 +853,7 @@ static void setupLuaRecords()
   lua.writeFunction("pickwhashed", [](std::unordered_map<int, wiplist_t > ips) {
       vector<pair<int,ComboAddress> > conv;
 
+      conv.reserve(ips.size());
       for(auto& i : ips)
         conv.emplace_back(atoi(i.second[1].c_str()), ComboAddress(i.second[2]));
 
@@ -996,7 +1016,7 @@ std::vector<shared_ptr<DNSRecordContent>> luaSynth(const std::string& code, cons
         ret.push_back(DNSRecordContent::mastermake(qtype, QClass::IN, content_it ));
     }
   } catch(std::exception &e) {
-    g_log << Logger::Info << "Lua record ("<<query<<"|"<<QType(qtype).getName()<<") reported: " << e.what();
+    g_log << Logger::Info << "Lua record ("<<query<<"|"<<QType(qtype).toString()<<") reported: " << e.what();
     try {
       std::rethrow_if_nested(e);
       g_log<<endl;
