@@ -16,8 +16,14 @@
 #include "minicurl.hh"
 #endif
 
+#include "tcpiohandler.hh"
+
 StatBag S;
 
+// Vars below used by tcpiohandler.cc
+bool g_verbose = true;
+bool g_syslog = false;
+
 static bool hidettl = false;
 
 static string ttl(uint32_t ttl)
@@ -32,29 +38,20 @@ static void usage()
 {
   cerr << "sdig" << endl;
   cerr << "Syntax: sdig IP-ADDRESS-OR-DOH-URL PORT QNAME QTYPE "
-          "[dnssec] [ednssubnet SUBNET/MASK] [hidesoadetails] [hidettl] "
-          "[recurse] [showflags] [tcp] [xpf XPFDATA] [class CLASSNUM] "
+          "[dnssec] [ednssubnet SUBNET/MASK] [hidesoadetails] [hidettl] [recurse] [showflags] "
+          "[tcp] [dot] [insecure] [fastOpen] [subjectName name] [caStore file] [tlsProvider openssl|gnutls] "
+          "[xpf XPFDATA] [class CLASSNUM] "
           "[proxy UDP(0)/TCP(1) SOURCE-IP-ADDRESS-AND-PORT DESTINATION-IP-ADDRESS-AND-PORT]"
+          "dumpluaraw"
        << endl;
 }
 
-static const string nameForClass(uint16_t qclass, uint16_t qtype)
+static const string nameForClass(QClass qclass, uint16_t qtype)
 {
   if (qtype == QType::OPT)
     return "IN";
 
-  switch (qclass) {
-  case QClass::IN:
-    return "IN";
-  case QClass::CHAOS:
-    return "CHAOS";
-  case QClass::NONE:
-    return "NONE";
-  case QClass::ANY:
-    return "ANY";
-  default:
-    return string("CLASS") + std::to_string(qclass);
-  }
+  return qclass.toString();
 }
 
 static std::unordered_set<uint16_t> s_expectedIDs;
@@ -63,7 +60,7 @@ static void fillPacket(vector<uint8_t>& packet, const string& q, const string& t
                        bool dnssec, const boost::optional<Netmask> ednsnm,
                        bool recurse, uint16_t xpfcode, uint16_t xpfversion,
                        uint64_t xpfproto, char* xpfsrc, char* xpfdst,
-                       uint16_t qclass, uint16_t qid)
+                       QClass qclass, uint16_t qid)
 {
   DNSPacketWriter pw(packet, DNSName(q), DNSRecordContent::TypeToNumber(t), qclass);
 
@@ -106,7 +103,7 @@ static void fillPacket(vector<uint8_t>& packet, const string& q, const string& t
   pw.getHeader()->id = htons(qid);
 }
 
-static void printReply(const string& reply, bool showflags, bool hidesoadetails)
+static void printReply(const string& reply, bool showflags, bool hidesoadetails, bool dumpluaraw)
 {
   MOADNSParser mdp(false, reply);
   if (!s_expectedIDs.count(ntohs(mdp.d_header.id))) {
@@ -127,6 +124,10 @@ static void printReply(const string& reply, bool showflags, bool hidesoadetails)
     cout << i->first.d_place - 1 << "\t" << i->first.d_name.toString() << "\t"
          << nameForClass(i->first.d_class, i->first.d_type) << "\t"
          << DNSRecordContent::NumberToType(i->first.d_type);
+    if (dumpluaraw) {
+      cout<<"\t"<< makeLuaString(i->first.d_content->serialize(DNSName(), true))<<endl;
+      continue;
+    }
     if (i->first.d_class == QClass::IN) {
       if (i->first.d_type == QType::RRSIG) {
         string zoneRep = i->first.d_content->getZoneRepresentation();
@@ -195,18 +196,27 @@ static void printReply(const string& reply, bool showflags, bool hidesoadetails)
 
 int main(int argc, char** argv)
 try {
+  /* default timeout of 10s */
+  int timeout = 10;
   bool dnssec = false;
   bool recurse = false;
   bool tcp = false;
   bool showflags = false;
   bool hidesoadetails = false;
   bool doh = false;
+  bool dot = false;
+  bool fastOpen = false;
+  bool insecureDoT = false;
   bool fromstdin = false;
   boost::optional<Netmask> ednsnm;
   uint16_t xpfcode = 0, xpfversion = 0, xpfproto = 0;
   char *xpfsrc = NULL, *xpfdst = NULL;
-  uint16_t qclass = QClass::IN;
+  QClass qclass = QClass::IN;
   string proxyheader;
+  string subjectName;
+  string caStore;
+  string tlsProvider = "openssl";
+  bool dumpluaraw = false;
 
   for (int i = 1; i < argc; i++) {
     if ((string)argv[i] == "--help") {
@@ -231,24 +241,30 @@ try {
     for (int i = 5; i < argc; i++) {
       if (strcmp(argv[i], "dnssec") == 0)
         dnssec = true;
-      if (strcmp(argv[i], "recurse") == 0)
+      else if (strcmp(argv[i], "recurse") == 0)
         recurse = true;
-      if (strcmp(argv[i], "showflags") == 0)
+      else if (strcmp(argv[i], "showflags") == 0)
         showflags = true;
-      if (strcmp(argv[i], "hidesoadetails") == 0)
+      else if (strcmp(argv[i], "hidesoadetails") == 0)
         hidesoadetails = true;
-      if (strcmp(argv[i], "hidettl") == 0)
+      else if (strcmp(argv[i], "hidettl") == 0)
         hidettl = true;
-      if (strcmp(argv[i], "tcp") == 0)
+      else if (strcmp(argv[i], "tcp") == 0)
         tcp = true;
-      if (strcmp(argv[i], "ednssubnet") == 0) {
+      else if (strcmp(argv[i], "dot") == 0)
+        dot = true;
+      else if (strcmp(argv[i], "insecure") == 0)
+        insecureDoT = true;
+      else if (strcmp(argv[i], "fastOpen") == 0)
+        fastOpen = true;
+      else if (strcmp(argv[i], "ednssubnet") == 0) {
         if (argc < i + 2) {
           cerr << "ednssubnet needs an argument" << endl;
           exit(EXIT_FAILURE);
         }
         ednsnm = Netmask(argv[++i]);
       }
-      if (strcmp(argv[i], "xpf") == 0) {
+      else if (strcmp(argv[i], "xpf") == 0) {
         if (argc < i + 6) {
           cerr << "xpf needs five arguments" << endl;
           exit(EXIT_FAILURE);
@@ -259,14 +275,35 @@ try {
         xpfsrc = argv[++i];
         xpfdst = argv[++i];
       }
-      if (strcmp(argv[i], "class") == 0) {
+      else if (strcmp(argv[i], "class") == 0) {
         if (argc < i+2) {
           cerr << "class needs an argument"<<endl;
           exit(EXIT_FAILURE);
         }
         qclass = atoi(argv[++i]);
       }
-      if (strcmp(argv[i], "proxy") == 0) {
+      else if (strcmp(argv[i], "subjectName") == 0) {
+        if (argc < i + 2) {
+          cerr << "subjectName needs an argument"<<endl;
+          exit(EXIT_FAILURE);
+        }
+        subjectName = argv[++i];
+      }
+      else if (strcmp(argv[i], "caStore") == 0) {
+        if (argc < i + 2) {
+          cerr << "caStore needs an argument"<<endl;
+          exit(EXIT_FAILURE);
+        }
+        caStore = argv[++i];
+      }
+      else if (strcmp(argv[i], "tlsProvider") == 0) {
+        if (argc < i + 2) {
+          cerr << "tlsProvider needs an argument"<<endl;
+          exit(EXIT_FAILURE);
+        }
+        tlsProvider = argv[++i];
+      }
+      else if (strcmp(argv[i], "proxy") == 0) {
         if(argc < i+4) {
           cerr<<"proxy needs three arguments"<<endl;
           exit(EXIT_FAILURE);
@@ -276,9 +313,27 @@ try {
         ComboAddress dest(argv[++i]);
         proxyheader = makeProxyHeader(ptcp, src, dest, {});
       }
+      else if (strcmp(argv[i], "dumpluaraw") == 0) {
+        dumpluaraw = true;
+      }
+      else {
+        cerr << argv[i] << ": unknown argument" << endl;
+        exit(EXIT_FAILURE);
+      }
     }
   }
 
+  if (dot) {
+    tcp = true;
+  }
+
+#ifndef HAVE_DNS_OVER_TLS
+  if (dot) {
+    cerr << "DoT requested but not compiled in" << endl;
+    exit(EXIT_FAILURE);
+  }
+#endif
+
   string reply;
   ComboAddress dest;
   if (*argv[1] == 'h') {
@@ -319,8 +374,8 @@ try {
     mch.insert(std::make_pair("Accept", "application/dns-message"));
     string question(packet.begin(), packet.end());
     // FIXME: how do we use proxyheader here?
-    reply = mc.postURL(argv[1], question, mch);
-    printReply(reply, showflags, hidesoadetails);
+    reply = mc.postURL(argv[1], question, mch, timeout, fastOpen);
+    printReply(reply, showflags, hidesoadetails, dumpluaraw);
 #else
     throw PDNSException("please link sdig against libcurl for DoH support");
 #endif
@@ -342,12 +397,26 @@ try {
       reply = reply.substr(2);
     }
 
-    printReply(reply, showflags, hidesoadetails);
+    printReply(reply, showflags, hidesoadetails, dumpluaraw);
   } else if (tcp) {
+    std::shared_ptr<TLSCtx> tlsCtx{nullptr};
+    if (dot) {
+      TLSContextParameters tlsParams;
+      tlsParams.d_provider = tlsProvider;
+      tlsParams.d_validateCertificates = !insecureDoT;
+      tlsParams.d_caStore = caStore;
+      tlsCtx = getTLSContext(tlsParams);
+    }
     uint16_t counter = 0;
     Socket sock(dest.sin4.sin_family, SOCK_STREAM);
-    sock.connect(dest);
-    sock.writen(proxyheader);
+    setTCPNoDelay(sock.getHandle()); // disable NAGLE, which does not play nicely with delayed ACKs
+    TCPIOHandler handler(subjectName, sock.releaseHandle(), timeout, tlsCtx, time(nullptr));
+    handler.connect(fastOpen, dest, timeout);
+    // we are writing the proxyheader inside the TLS connection. Is that right?
+    if (proxyheader.size() > 0 && handler.write(proxyheader.data(), proxyheader.size(), timeout) != proxyheader.size()) {
+      throw PDNSException("tcp write failed");
+    }
+
     for (const auto& it : questions) {
       vector<uint8_t> packet;
       s_expectedIDs.insert(counter);
@@ -355,31 +424,28 @@ try {
                  xpfversion, xpfproto, xpfsrc, xpfdst, qclass, counter);
       counter++;
 
-      uint16_t len = htons(packet.size());
-      if (sock.write((const char *)&len, 2) != 2)
+      // Prefer to do a single write, so that fastopen can send all the data on SYN
+      uint16_t len = packet.size();
+      string question;
+      question.reserve(sizeof(len) + packet.size());
+      question.push_back(static_cast<char>(len >> 8));
+      question.push_back(static_cast<char>(len & 0xff));
+      question.append(packet.begin(), packet.end());
+      if (handler.write(question.data(), question.size(), timeout) != question.size()) {
         throw PDNSException("tcp write failed");
-      string question(packet.begin(), packet.end());
-      sock.writen(question);
+      }
     }
     for (size_t i = 0; i < questions.size(); i++) {
       uint16_t len;
-      if (sock.read((char *)&len, 2) != 2)
+      if (handler.read((char *)&len, sizeof(len), timeout) != sizeof(len)) {
         throw PDNSException("tcp read failed");
-
-      len = ntohs(len);
-      char* creply = new char[len];
-      int n = 0;
-      int numread;
-      while (n < len) {
-        numread = sock.read(creply + n, len - n);
-        if (numread < 0)
-          throw PDNSException("tcp read failed");
-        n += numread;
       }
-
-      reply = string(creply, len);
-      delete[] creply;
-      printReply(reply, showflags, hidesoadetails);
+      len = ntohs(len);
+      reply.resize(len);
+      if (handler.read(&reply[0], len, timeout) != len) {
+        throw PDNSException("tcp read failed");
+      }
+      printReply(reply, showflags, hidesoadetails, dumpluaraw);
     }
   } else // udp
   {
@@ -391,13 +457,13 @@ try {
     Socket sock(dest.sin4.sin_family, SOCK_DGRAM);
     question = proxyheader + question;
     sock.sendTo(question, dest);
-    int result = waitForData(sock.getHandle(), 10);
+    int result = waitForData(sock.getHandle(), timeout);
     if (result < 0)
       throw std::runtime_error("Error waiting for data: " + stringerror());
     if (!result)
       throw std::runtime_error("Timeout waiting for data");
     sock.recvFrom(reply, dest);
-    printReply(reply, showflags, hidesoadetails);
+    printReply(reply, showflags, hidesoadetails, dumpluaraw);
   }
 
 } catch (std::exception& e) {
